/*!
 * rivet-core - @version 2.0.0-alpha

 * Copyright (C) 2018 The Trustees of Indiana University
 * SPDX-License-Identifier: BSD-3-Clause
 */

if (!Element.prototype.closest) {
  if (!Element.prototype.matches) {
    Element.prototype.matches = Element.prototype.msMatchesSelector || Element.prototype.webkitMatchesSelector;
  }

  Element.prototype.closest = function (s) {
    var el = this;
    var ancestor = this;
    if (!document.documentElement.contains(el)) return null;

    do {
      if (ancestor.matches(s)) return ancestor;
      ancestor = ancestor.parentElement;
    } while (ancestor !== null);

    return null;
  };
}

(function () {
  if (typeof window.CustomEvent === 'function') return false;

  function CustomEvent(event, params) {
    params = params || {
      bubbles: false,
      cancelable: false,
      detail: undefined
    };
    var evt = document.createEvent('CustomEvent');
    evt.initCustomEvent(event, params.bubbles, params.cancelable, params.detail);
    return evt;
  }

  CustomEvent.prototype = window.Event.prototype;
  window.CustomEvent = CustomEvent;
})();

(function (arr) {
  arr.forEach(function (item) {
    if (item.hasOwnProperty('remove')) {
      return;
    }


    Object.defineProperty(item, 'remove', {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function remove() {
        if (this.parentNode === null) {
          return;
        }

        this.parentNode.removeChild(this);
      }
    });
  });
})([Element.prototype, CharacterData.prototype, DocumentType.prototype]);

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(source, true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(source).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

var globalSettings = {
  prefix: 'rvt'
};

var prefix = globalSettings.prefix;

function dispatchCustomEvent(eventName, element, detail) {
  var newEvent = new CustomEvent("".concat(prefix, ":").concat(eventName), {
    bubbles: true,
    cancelable: true,
    detail: detail
  });
  var eventShouldFire = element.dispatchEvent(newEvent);
  return eventShouldFire;
}


var nodeListToArray = function nodeListToArray(nodes) {
  return Array.prototype.slice.call(nodes);
};

var isNode = function isNode(element) {
  return element.nodeType === 1 || element.nodeType === 1;
};

var Alert =
function () {
  function Alert(element) {
    _classCallCheck(this, Alert);

    this.element = element;
    this.closeAttribute = 'data-alert-close';
    this.closeSelector = "[".concat(this.closeAttribute, "]");
    this.closeButton = null; 

    this._handleClick = this._handleClick.bind(this); 

    if (!isNode(this.element)) {
      throw new TypeError('A DOM element should be passed as the first argument to initialize the alert');
    }

    this.init();
  }

  _createClass(Alert, [{
    key: "_handleClick",
    value: function _handleClick() {
      this.dismiss();
    }
  }, {
    key: "dismiss",
    value: function dismiss() {
      var dismissEvent = dispatchCustomEvent('alertDismiss', this.element, {
        id: this.element.dataset.alert
      });
      if (!dismissEvent) return;
      this.destroy();
      this.element.remove();
    }
  }, {
    key: "init",
    value: function init() {
      this.closeButton = this.element.querySelector(this.closeSelector);
      this.closeButton.addEventListener('click', this._handleClick, false);
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.closeButton.addEventListener('click', this._handleClick, false);
    }
  }]);

  return Alert;
}();

var FileInput =
function () {
  function FileInput(element, options) {
    _classCallCheck(this, FileInput);

    var defaultOptions = {
      previewText: 'No file selected'
    };

    var settings = _objectSpread2({}, defaultOptions, {}, options);

    this.element = element;
    this.wrapperAttribute = 'data-upload-wrapper';
    this.inputAttribute = 'data-upload-input';
    this.previewAttribute = 'data-upload-preview';
    this.previewText = settings.previewText;
    this._handleChange = this._handleChange.bind(this);
    this.init();
  }


  _createClass(FileInput, [{
    key: "_sanitizeHTML",
    value: function _sanitizeHTML(str) {
      var temp = document.createElement('div');
      temp.textContent = str;
      return temp.innerHTML;
    }

  }, {
    key: "_buildSingleFile",
    value: function _buildSingleFile(input) {
      var singleFileItem = document.createElement('span');

      var singleFileName = this._sanitizeHTML(input.files[0].name); 


      singleFileItem.textContent = singleFileName; 

      return singleFileItem;
    }

  }, {
    key: "_buildMultipleFiles",
    value: function _buildMultipleFiles(input) {
      var fileCount = document.createElement('span');
      fileCount.textContent = input.files.length + ' files selected';
      return fileCount;
    }
  }, {
    key: "_handleChange",
    value: function _handleChange() {
      var uploadInput = this.element.querySelector("[".concat(this.inputAttribute, "]")); 

      var uploadPreview = this.element.querySelector("[".concat(this.previewAttribute, "]")); 

      if (uploadInput.files.length > 0) {
        uploadPreview.innerHTML = '';

        uploadInput.files.length > 1 ? uploadPreview.appendChild(this._buildMultipleFiles(uploadInput)) : uploadPreview.appendChild(this._buildSingleFile(uploadInput));
      } else {
        uploadPreview.innerHTML = this.previewText;
      }
    }
  }, {
    key: "init",
    value: function init() {
      this.element.addEventListener('change', this._handleChange, false);
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.element.removeEventListener('change', this._handleChange, false);
    }
  }]);

  return FileInput;
}();

var Sidenav =
function () {
  function Sidenav(element, options) {
    _classCallCheck(this, Sidenav);

    var defaultOptions = {
      openAllOnInit: false
    };

    var settings = _objectSpread2({}, defaultOptions, {}, options); 


    this.element = element;
    this.openAllOnInit = settings.openAllOnInit;
    this.toggleAttribute = 'data-sidenav-toggle', this.toggleSelector = "[".concat(this.toggleAttribute, "]");
    this.listAttribute = 'data-sidenav-list';
    this.listSelector = "[".concat(this.listAttribute, "]"); 

    this._handleClick = this._handleClick.bind(this); 

    if (!isNode(this.element)) {
      throw new TypeError('A DOM element should be passed as the first argument to initialize the sidenav');
    }

    this.init();
  }

  _createClass(Sidenav, [{
    key: "_handleClick",
    value: function _handleClick(event) {
      var toggleButton = event.target.closest(this.toggleSelector); 

      if (!toggleButton) return;
      var toggleId = toggleButton.dataset.sidenavToggle;
      var targetList = this.element.querySelector("[".concat(this.listAttribute, "=\"").concat(toggleId, "\"]")); 

      if (!targetList || targetList.getAttribute(this.listAttribute) === '') {
        return;
      }

      targetList.hasAttribute('hidden') ? this.open(toggleButton, targetList) : this.close(toggleButton, targetList);
    }
  }, {
    key: "open",
    value: function open(toggleButton, targetList) {
      var openEvent = dispatchCustomEvent('sidenavListOpen', toggleButton, {
        id: toggleButton.dataset.sidenavToggle
      });
      if (!openEvent) return;
      toggleButton.setAttribute('aria-expanded', 'true');
      targetList.removeAttribute('hidden');
    }
  }, {
    key: "close",
    value: function close(toggleButton, targetList) {
      var closeEvent = dispatchCustomEvent('sidenavListClose', toggleButton, {
        id: toggleButton.dataset.sidenavToggle
      });
      if (!closeEvent) return;
      toggleButton.setAttribute('aria-expanded', 'false');
      targetList.setAttribute('hidden', '');
    }
  }, {
    key: "init",
    value: function init() {
      var _this = this;

      var menuToggles = nodeListToArray(this.element.querySelectorAll(this.toggleSelector));
      var childMenus = nodeListToArray(this.element.querySelectorAll(this.listSelector));
      menuToggles.forEach(function (menuToggle) {
        menuToggle.setAttribute('aria-haspopup', 'true'); 

        if (!_this.openAllOnInit) {
          menuToggle.setAttribute('aria-expanded', 'false');
        }
      }); 

      if (!this.openAllOnInit) {
        childMenus.forEach(function (childMenu) {
          return childMenu.setAttribute('hidden', '');
        });
      } 


      this.element.addEventListener('click', this._handleClick, false);
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.element.removeEventListener('click', this._handleClick, false);
    }
  }]);

  return Sidenav;
}();

var index = {
  Alert: Alert,
  FileInput: FileInput,
  Sidenav: Sidenav
};

export default index;
